@@ -1,10 +1,20 @@
 package docker
 
 import (
+	"slices"
+	"strings"
+
+	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/volumes"
 )
 
 var createVolumesManager = func(e *executor) (volumes.Manager, error) {
+	// Note if any of the cache keys includes the `-protected` suffix (but not the `-non_protected` suffix).
+	// See https://gitlab.com/gitlab-org/gitlab/-/work_items/494478.
+	protectedKeyIdx := slices.IndexFunc(e.Build.Cache, func(c common.Cache) bool {
+		return strings.HasSuffix(c.Key, "-protected") && !strings.HasSuffix(c.Key, "-non_protected")
+	})
+
 	config := volumes.ManagerConfig{
 		CacheDir:      e.Config.Docker.CacheDir,
 		BasePath:      e.Build.FullProjectDir(),
@@ -13,7 +23,9 @@ var createVolumesManager = func(e *executor) (volumes.Manager, error) {
 		DisableCache:  e.Config.Docker.DisableCache,
 		Driver:        e.Config.Docker.VolumeDriver,
 		DriverOpts:    e.Config.Docker.VolumeDriverOps,
-		Protected:     e.Build.IsProtected(),
+		// the volume should be protected if the ref is protected OR if any of the cache volumes have the protected
+		// suffix. See https://gitlab.com/gitlab-org/gitlab/-/work_items/494478.
+		Protected: e.Build.IsProtected() || protectedKeyIdx >= 0,
 	}
 
 	if e.newVolumePermissionSetter != nil {
