@@ -44,11 +44,12 @@ var dialer = net.Dialer{
 	KeepAlive: 30 * time.Second,
 }
 
-type Option = func(c *client)
+type option func(*client)
 
 type client struct {
 	http.Client
 	url              *url.URL
+	certDirectory    string
 	caFile           string
 	certFile         string
 	keyFile          string
@@ -519,23 +520,29 @@ func fixCIURL(url string) string {
 	return url
 }
 
-func (n *client) findCertificate(certificate *string, base string, name string) {
+func (n *client) findCertificate(certificate *string, name string) {
 	if *certificate != "" {
 		return
 	}
-	path := filepath.Join(base, name)
+	path := filepath.Join(n.certDirectory, name)
 	if _, err := os.Stat(path); err == nil {
 		*certificate = path
 	}
 }
 
-func WithMaxAge(connectionMaxAge time.Duration) Option {
+func withMaxAge(connectionMaxAge time.Duration) option {
 	return func(c *client) {
 		c.connectionMaxAge = connectionMaxAge
 	}
 }
 
-func newClient(requestCredentials requestCredentials, collector *APIRequestsCollector, options ...Option) (*client, error) {
+func withCertificateDirectory(certDirectory string) option {
+	return func(c *client) {
+		c.certDirectory = certDirectory
+	}
+}
+
+func newClient(requestCredentials requestCredentials, collector *APIRequestsCollector, options ...option) (*client, error) {
 	url, err := url.Parse(fixCIURL(requestCredentials.GetURL()) + "/api/v4/")
 	if err != nil {
 		return nil, fmt.Errorf("parse URL: %w", err)
@@ -553,16 +560,16 @@ func newClient(requestCredentials requestCredentials, collector *APIRequestsColl
 	}
 	c.requester = newRetryRequester(&c.Client, collector)
 
-	host := strings.Split(url.Host, ":")[0]
-	if CertificateDirectory != "" {
-		c.findCertificate(&c.caFile, CertificateDirectory, host+".crt")
-		c.findCertificate(&c.certFile, CertificateDirectory, host+".auth.crt")
-		c.findCertificate(&c.keyFile, CertificateDirectory, host+".auth.key")
-	}
-
 	for _, o := range options {
 		o(c)
 	}
 
+	if c.certDirectory != "" {
+		host := strings.Split(url.Host, ":")[0]
+		c.findCertificate(&c.caFile, host+".crt")
+		c.findCertificate(&c.certFile, host+".auth.crt")
+		c.findCertificate(&c.keyFile, host+".auth.key")
+	}
+
 	return c, nil
 }
